///--------------------------------------------------------------------------------------
// Scrolling Demo.c - Modified version
//
// By Vern Jensen. Created sometime in 1997
///--------------------------------------------------------------------------------------


#include <SWFPSReport.h>
#include <SWIncludes.h>
#include <SWGameUtils.h>

#include "SWApplication.h"
#include "Scrolling Demo.h"

#define kMaxWindowWidth				640			// User can expand window to this size
#define kMaxWindowHeight			480

#define	kInterlacedMode				false		// Turns Interlaced mode on/off
#define kSyncToVBL					false		// Sync SpriteWorld to VBL?
#define kMaxFPS						30			// Set to 0 for unrestricted speed
	
#define	kSpriteMoveDelta			20			// Try 5, 10, 20, or 40
#define kDiamondSpace				8			// How far apart the diamonds are spaced
												// (try 1!)

#define kSpriteMoveDistance			80			// How far the sprite can move from
												// the center of the screen, in pixels.
												// Try making this value higher!

	// Number of ticks to wait before changing tile image; 0 = change every frame
#define kDiamondFrameRate			8			// How often diamond tiles change frames
#define kWallFrameRate				120			// How often wall changes frames

#define kBWPictResIDOffset			100
#define kTileWidth					40
#define kTileHeight					40
#define kTileMapRows				100
#define kTileMapCols				150

#define kStartRow					5			// Starting position of sprite
#define kStartCol					5			// in tile col and row


#define	kLeftArrowKey				0x7B
#define	kRightArrowKey				0x7C
#define	kDownArrowKey				0x7D
#define	kUpArrowKey					0x7E

#define	kLeftKeyPad					0x56
#define	kRightKeyPad				0x58
#define	kDownKeyPad					0x54
#define	kUpKeyPad					0x5B

#define kEscKey						0x35


#define kNoKey						0
#define kLeftKey					1
#define kUpKey						2
#define kRightKey					3
#define kDownKey					4


enum tileIDs
{
	kWallTile,
	kLastWallTile,
	kGrassTile,
	kBlackTile,
	kDiamondTile,
	kDiamondTile2,
	kLastDiamondTile,
	
	kTunnelTile1,
	kTunnelTile2,
	kTunnelTile3,
	kTunnelTile4,
	kTunnelTile5,
	kTunnelTile6,
	kWireTile1,
	kWireTile2,
	
	kMaxNumTiles
};



/***********/
/* Globals */
/***********/

SpriteWorldPtr		gSpriteWorldP;
SpriteLayerPtr		gSpriteLayerP;
SpriteLayerPtr		gMouseSpriteLayerP;
TileMapStructPtr	gTileMapStructP;
TileMapPtr			gTileMap;
SpritePtr			gSimpleSpriteP, gDiamondMeterSpriteP, gMouseSpriteP;
DrawProcPtr			gSpriteDrawProc;
DrawProcPtr			gScreenDrawProc;
DoubleDrawProcPtr	gDoubleRectDrawProc;
WindowPtr			gWindowP;
Rect				gScreenMidRect;
Boolean				gDone = false;

short				gNumDiamondsInMap = 0;	// Number of diamonds in the TileMap
short				gNumDiamonds = 0;		// Number of diamonds sprite has collected

struct moveKeys		// Keeps track of which keys are up and which are down
{
	Boolean	up;
	Boolean	right;
	Boolean	down;
	Boolean	left;
} gKeys;


///--------------------------------------------------------------------------------------
// Main
///--------------------------------------------------------------------------------------

void	main( void )
{
	Initialize(kNumberOfMoreMastersCalls);
	
	if (SWHasSystem7())
	{
		AllowKeyUpEvents();	// Part of SWGameUtils.c
		SetCursor(*GetCursor(watchCursor));
		
		CreateSpriteWorld();
		CreateBallSprite();
		CreateDiamondMeterSprite();
		CreateMouseSprite();
		
		SetCursor(&qd.arrow);
		HideCursor();
		
		SetUpAnimation();
		RunAnimation();
		ShutDown();
		
		RestoreEventMask();	// Call this after AllowKeyUpEvents
	}
	else
	{
		CantRunOnThisMachine();
	}
}


///--------------------------------------------------------------------------------------
// CreateSpriteWorld
///--------------------------------------------------------------------------------------

void	CreateSpriteWorld( void )
{
	Rect		offscreenRect, worldRect, windRect;
	OSErr		err;
	short		row, col;
	short		resIDOffset;
	WindowPeek	windowPeek;
	WStateData	*wStateData;
	
	gWindowP = GetNewCWindow(129, NULL, (WindowPtr)-1L);
	
	if (gWindowP != NULL)
	{
			// Center window in screen
		windRect = gWindowP->portRect;
		if (qd.screenBits.bounds.bottom < 440 || qd.screenBits.bounds.right < 600)
		{
				// We're running in 512x384, so make the window smaller so it can be moved
			windRect.bottom -= 80;
			windRect.right -= 80;
			SizeWindow(gWindowP, windRect.right - windRect.left, 
						windRect.bottom - windRect.top, false);
		}
		CenterRect(&windRect, &qd.screenBits.bounds);
		windRect.top += 10;
		windRect.bottom += 10;
		
		AlignRect(&windRect);
		MoveWindow(gWindowP, windRect.left, windRect.top, false);
		
		ShowWindow(gWindowP);
		SetPort(gWindowP);
		
		if (kInterlacedMode)
			PaintRect(&gWindowP->portRect);	// Blacken window for Interlaced mode
	}
	else
		CantFindResource();
	
		// Change window's standard size to its current size. (Used when zooming window.)
	windowPeek = (WindowPeek)gWindowP;
	wStateData = (WStateData *) *(windowPeek->dataHandle);
	wStateData->stdState = (**(windowPeek->contRgn)).rgnBBox;

	
	err = SWEnterSpriteWorld();
	FatalError(err);
	
	
	worldRect = gWindowP->portRect;
	
		// Set size of offscreen area
	offscreenRect = worldRect;
	OffsetRect(&offscreenRect, -offscreenRect.left, -offscreenRect.top);
	offscreenRect.right = SW_MAX(offscreenRect.right, kMaxWindowWidth);
	offscreenRect.bottom = SW_MAX(offscreenRect.bottom, kMaxWindowHeight);

		// Make offscreen area evenly divisible by tile width & height
	if ( (offscreenRect.right/kTileWidth)*kTileWidth != offscreenRect.right)
		offscreenRect.right = (offscreenRect.right/kTileWidth)*kTileWidth + kTileWidth;
	
	if ( (offscreenRect.bottom/kTileHeight)*kTileHeight != offscreenRect.bottom)
		offscreenRect.bottom = (offscreenRect.bottom/kTileHeight)*kTileHeight + kTileHeight;
	
		// Create the scrolling sprite world
	err = SWCreateSpriteWorldFromWindow(&gSpriteWorldP, (CWindowPtr)gWindowP, 
			&worldRect, &offscreenRect, 0);
	FatalError(err);
	
	
			// Create the sprite layers
	err = SWCreateSpriteLayer(&gSpriteLayerP);
	FatalError(err);
	err = SWCreateSpriteLayer(&gMouseSpriteLayerP);
	FatalError(err);
	
		// add the layers to the world
	SWAddSpriteLayer(gSpriteWorldP, gSpriteLayerP);			// Bottom layer
	SWAddSpriteLayer(gSpriteWorldP, gMouseSpriteLayerP);	// Top layer


	err = SWInitTiling(gSpriteWorldP, kTileHeight, kTileWidth, kMaxNumTiles);
	FatalError(err);
	
	err = SWCreateTileMap(&gTileMapStructP, kTileMapRows, kTileMapCols);
	FatalError(err);
	
	SWInstallTileMap(gSpriteWorldP, gTileMapStructP, 0);
	gTileMap = gTileMapStructP->tileMap;
	
	SWSetSpriteLayerUnderTileLayer(gSpriteLayerP, 0);
	

		// Determine whether to load B&W or color graphics
	if (gSpriteWorldP->pixelDepth <= 2)
		resIDOffset = kBWPictResIDOffset;
	else
		resIDOffset = 0;

		// Load first set of tiles
	err = SWLoadTilesFromPictResource(
		gSpriteWorldP, 
		kWallTile,				// startTileID 
		kLastDiamondTile,		// endTileID
		200 + resIDOffset,		// pictResID
		0,						// maskResID
		kNoMask,				// maskType
		1,						// horizBorderWidth
		1);						// vertBorderHeight
	FatalError(err);
	
		// Load masked set of tiles
	err = SWLoadTilesFromPictResource(
		gSpriteWorldP, 
		kTunnelTile1,			// startTileID 
		kWireTile2,				// endTileID
		201 + resIDOffset,		// pictResID
		401,					// maskResID
		kFatMask,				// maskType
		1,						// horizBorderWidth
		1);						// vertBorderHeight
	FatalError(err);

	
		// Set up tileMap
	for (row = 0; row < kTileMapRows; row++)
	{
		for (col = 0; col < kTileMapCols; col++)
		{
			if (row == 0 || col == 0 || row == kTileMapRows-1 || col == kTileMapCols-1)
				gTileMap[row][col] = kWallTile;
			else if (row > kTileMapRows / 2)
				gTileMap[row][col] = kWireTile1;
			else if ((row/kDiamondSpace)*kDiamondSpace == row && 
					 (col/kDiamondSpace)*kDiamondSpace == col)
			{
				gTileMap[row][col] = kDiamondTile;
				gNumDiamondsInMap++;
			}
			else
				gTileMap[row][col] = kGrassTile;
		}
	}
	

		// Add tunnel to tileMap
	row = kTileMapRows / 2;

	for (col = 1; col < kTileMapCols-1; col += 2)
	{
		gTileMap[row][col] = kTunnelTile1;
		gTileMap[row][col+1] = kTunnelTile2;
		gTileMap[row+1][col] = kTunnelTile4;
		gTileMap[row+1][col+1] = kTunnelTile5;
	}
	
	
		// Determine what DrawProcs to use
	if (gSpriteWorldP->pixelDepth == 8)		// 256 colors
	{
		if (kInterlacedMode)
		{
			gSpriteDrawProc = BP8BitInterlacedMaskDrawProc;
			gScreenDrawProc = BP8BitInterlacedRectDrawProc;
			gDoubleRectDrawProc = BP8BitInterlacedDoubleRectDrawProc;
			SWSetTileMaskDrawProc(gSpriteWorldP, BP8BitInterlacedPartialMaskDrawProc);
		}
		else
		{
			gSpriteDrawProc = BlitPixie8BitMaskDrawProc;
			gScreenDrawProc = BlitPixie8BitRectDrawProc;
			gDoubleRectDrawProc = BlitPixie8BitDoubleRectDrawProc;
			SWSetTileMaskDrawProc(gSpriteWorldP, BlitPixie8BitPartialMaskDrawProc);
		}
	}
	else if ( !(SW_PPC && gSpriteWorldP->pixelDepth < 8) )	// Not 256 colors
	{
			// Use interlaced drawProcs unless in B&W, where interlacing is ugly
		if (kInterlacedMode && gSpriteWorldP->pixelDepth > 2)
		{
			gSpriteDrawProc = BPAllBitInterlacedMaskDrawProc;
			gScreenDrawProc = BPAllBitInterlacedRectDrawProc;
			SWSetTileMaskDrawProc(gSpriteWorldP, BPAllBitInterlacedPartialMaskDrawProc);
			if (gSpriteWorldP->pixelDepth == 16)
				gDoubleRectDrawProc = BP16BitInterlacedDoubleRectDrawProc;
		}
		else
		{
			gSpriteDrawProc = BlitPixieAllBitMaskDrawProc;
			gScreenDrawProc = BlitPixieAllBitRectDrawProc;
			SWSetTileMaskDrawProc(gSpriteWorldP, BlitPixieAllBitPartialMaskDrawProc);
			if (gSpriteWorldP->pixelDepth == 16)
				gDoubleRectDrawProc = BlitPixie16BitDoubleRectDrawProc;
		}
	}
	else
	{
		gSpriteDrawProc = SWStdSpriteDrawProc;
		gScreenDrawProc = SWStdWorldDrawProc;
		gDoubleRectDrawProc = NULL;
	}
}
	

///--------------------------------------------------------------------------------------
// CreateBallSprite
///--------------------------------------------------------------------------------------

void	CreateBallSprite( void )
{
	OSErr				err;
	
		// Create the ball sprite
	err = SWCreateSpriteFromCicnResource(gSpriteWorldP, &gSimpleSpriteP, NULL, 
			128, 1, kFatMask);	
	FatalError(err);
	
		// Set up the ball sprite
	SWAddSprite(gSpriteLayerP, gSimpleSpriteP);
	SWSetSpriteMoveProc(gSimpleSpriteP, KeySpriteMoveProc);
	SWSetSpriteLocation(gSimpleSpriteP, kStartCol * kTileWidth, kStartRow * kTileHeight);
	SWSetSpriteMoveDelta(gSimpleSpriteP, 0, 0);
	SWSetSpriteDrawProc(gSimpleSpriteP, gSpriteDrawProc);
}


///--------------------------------------------------------------------------------------
// CreateDiamondMeterSprite
///--------------------------------------------------------------------------------------

void	CreateDiamondMeterSprite( void )
{
	short				spriteResID;
	OSErr				err;
	
		// Calculate resource ID for diamond meter sprite
	if (gSpriteWorldP->pixelDepth <= 2)
		spriteResID = 302;
	else
		spriteResID = 202;
	
		// Create the diamond meter sprite
	err = SWCreateSpriteFromPictResource(gSpriteWorldP,
				&gDiamondMeterSpriteP, 
				NULL,			// pointer to memory for sprite
				spriteResID, 	// picture resource id
				402,			// mask resource id
				1, 				// max frames
				kPixelMask);		// mask type
	FatalError(err);
	
	SWAddSprite(gSpriteLayerP, gDiamondMeterSpriteP);
	SWSetSpriteDrawProc(gDiamondMeterSpriteP, gSpriteDrawProc);
}


///--------------------------------------------------------------------------------------
// CreateMouseSprite
///--------------------------------------------------------------------------------------

void	CreateMouseSprite( void )
{
	OSErr				err;
	
		// Create the mouse sprite
	err = SWCreateSpriteFromCicnResource(gSpriteWorldP, &gMouseSpriteP, NULL, 
			130, 1, kPixelMask);	
	FatalError(err);

	SWAddSprite(gMouseSpriteLayerP, gMouseSpriteP);
	SWSetSpriteDrawProc(gMouseSpriteP, gSpriteDrawProc);
}


///--------------------------------------------------------------------------------------
// SetUpAnimation
///--------------------------------------------------------------------------------------

void	SetUpAnimation( void )
{
	Rect		moveBoundsRect, worldRect;
	
	SWLockSpriteWorld(gSpriteWorldP);
	
		// Set up data used by the SmoothScrollingWorldMoveProc
	gScreenMidRect = gSimpleSpriteP->curFrameP->frameRect;
	worldRect = gSpriteWorldP->windRect;
	OffsetRect(&worldRect, -worldRect.left, -worldRect.top);
	CenterRect( &gScreenMidRect, &worldRect );
	
	SWSetSpriteWorldMaxFPS(gSpriteWorldP, kMaxFPS);
	SWSyncSpriteWorldToVBL(gSpriteWorldP, kSyncToVBL);
	
		// movement boundary = size of tileMap
	SetRect(&moveBoundsRect, 0,0, kTileMapCols * kTileWidth, kTileMapRows * kTileHeight);
	
	SWSetScrollingWorldMoveBounds(gSpriteWorldP, &moveBoundsRect);
	SWSetScrollingWorldMoveProc(gSpriteWorldP, SmoothScrollingWorldMoveProc, gSimpleSpriteP);
	
	SWSetTileChangeProc(gSpriteWorldP, TileChangeProc);
	
		// Move visScrollRect to starting sprite position
	SWMoveVisScrollRect(gSpriteWorldP, 
		gSpriteWorldP->followSpriteP->destFrameRect.left - gSpriteWorldP->backRect.right/2,
		gSpriteWorldP->followSpriteP->destFrameRect.top - gSpriteWorldP->backRect.bottom/2);
	
		// Set starting position of diamond meter sprite
	DiamondMeterSpriteMoveProc(gDiamondMeterSpriteP);

	SWSetSpriteWorldScreenDrawProc(gSpriteWorldP, gScreenDrawProc);
	SWSetSpriteWorldOffscreenDrawProc(gSpriteWorldP, gScreenDrawProc);
	SWSetDoubleRectDrawProc(gSpriteWorldP, gDoubleRectDrawProc);
	
		// Make sure CopyBits, if used, doesn't try to colorize things
	SWSetPortToWindow(gSpriteWorldP);
	ForeColor(blackColor);
	BackColor(whiteColor);
	
	SWSetPortToWorkArea(gSpriteWorldP);
	ForeColor(blackColor);
	PaintRect(&gSpriteWorldP->backRect);
	SWDrawTilesInBackground(gSpriteWorldP);
	SWUpdateScrollingSpriteWorld(gSpriteWorldP, true);
	ShowCursor();
}


///--------------------------------------------------------------------------------------
//  RunAnimation
///--------------------------------------------------------------------------------------

void	RunAnimation( void )
{
	EventRecord			event;
	unsigned long		frames;
	
	frames = 0;
	StartTimer();

	while (!gDone)
	{
		if ( WaitNextEvent(everyEvent, &event, 0L, NULL) )
			DispatchEvent(&event);

			// Make sure window wasn't "rolled up" with WindowShade
		if ( !EmptyRgn( gWindowP->visRgn ) )
		{
			SWProcessScrollingSpriteWorld(gSpriteWorldP);
	
				// Move diamond meter & mouse sprites to new visScrollRect location
			DiamondMeterSpriteMoveProc(gDiamondMeterSpriteP);
			MouseSpriteMoveProc(gMouseSpriteP);
				
			SWAnimateScrollingSpriteWorld(gSpriteWorldP);
			
			if (gSpriteWorldP->frameHasOccurred)
				frames++;
		}
	}
	
	ShowResults(frames);
}

///--------------------------------------------------------------------------------------
//  DispatchEvent
///--------------------------------------------------------------------------------------

void DispatchEvent(EventRecord* event)
{
	short			theKey, theChar;
	Boolean			isDown;
	
	switch(event->what)
	{
		case mouseDown:
			HandleMouseDown( event );
			break;
		case mouseUp:
			break;
		case keyUp:
		case keyDown:
		case autoKey:
			theKey = (event->message & keyCodeMask) >> 8;
			theChar = event->message & charCodeMask;
			isDown = (event->what != keyUp);
			
			if ( (theKey == kLeftArrowKey) || (theKey == kLeftKeyPad) )
				gKeys.left = isDown;
			else if ( (theKey == kRightArrowKey) || (theKey == kRightKeyPad) )
				gKeys.right = isDown;
			else if ( (theKey == kDownArrowKey) || (theKey == kDownKeyPad) )
				gKeys.down = isDown;
			else if ( (theKey == kUpArrowKey) || (theKey == kUpKeyPad) )
				gKeys.up = isDown;
			else if ( (theKey == kEscKey) || (theChar == 'Q') || (theChar == 'q') )
				gDone = true;
			break;
			break;
		case updateEvt:
			break;
		case diskEvt:
			break;
		case activateEvt:
			break;
		case networkEvt:
			break;
		case driverEvt:
			break;
		case app1Evt:
			break;
		case app2Evt:
			break;
		case app3Evt:
			break;
		case osEvt:
			if ( ( event->message & suspendResumeMessage ) == resumeFlag )
			{
				SWSetSpriteWorldScreenDrawProc(gSpriteWorldP, gScreenDrawProc);
				SWSetDoubleRectDrawProc(gSpriteWorldP, gDoubleRectDrawProc);
			}
			else
			{
				SWSetSpriteWorldScreenDrawProc(gSpriteWorldP, SWStdWorldDrawProc);
				SWSetDoubleRectDrawProc(gSpriteWorldP, NULL);
			}
			break;
		case kHighLevelEvent:
			break;
		default:
		break;
	}
}


///--------------------------------------------------------------------------------------
//  HandleMouseDown
///--------------------------------------------------------------------------------------

void	HandleMouseDown( EventRecord *eventPtr )
{
	WindowPtr		theWindow;
	Rect			windRect;
	short			thePart;
	Point			thePoint;
	long			windowSize;
	Rect			worldRect;
	
	
	thePoint = eventPtr->where;
	thePart = FindWindow( thePoint, &theWindow );

	switch ( thePart )
	{
		case inSysWindow : 
			SystemClick( eventPtr, theWindow );
			break;
		case inDrag:
			DragWindow( theWindow, thePoint, &((**(GetGrayRgn())).rgnBBox) );
			windRect = (*((WindowPeek)gWindowP)->contRgn)->rgnBBox;
			
			AlignRect(&windRect);
			MoveWindow(gWindowP, windRect.left, windRect.top, false);
			
			SWWindowMoved( gSpriteWorldP );
			break;
		case inZoomOut:
		case inZoomIn:
			if ( TrackBox(theWindow, thePoint, thePart) )
			{
				ZoomWindow(theWindow, thePart, true);
				SWChangeWorldRect( gSpriteWorldP, &theWindow->portRect, false);
				SWDrawTilesInBackground( gSpriteWorldP );
				SWUpdateScrollingSpriteWorld( gSpriteWorldP , false );
			
					// Reset data used by SmoothScrollingWorldMoveProc
				worldRect = gSpriteWorldP->windRect;
				OffsetRect(&worldRect, -worldRect.left, -worldRect.top);
				CenterRect( &gScreenMidRect, &worldRect );
			}
			break;
		case inMenuBar:
			break;
		case inGoAway:
			if ( TrackGoAway(theWindow, thePoint) )
				gDone = true;
			break;
		case inGrow:
			SetRect(&windRect, 80, 80, 
				gSpriteWorldP->originalBackRect.right - gSpriteWorldP->originalBackRect.left+1, 
				gSpriteWorldP->originalBackRect.bottom - gSpriteWorldP->originalBackRect.top+1); 
			windowSize = GrowWindow( theWindow, eventPtr->where, &windRect );
			SizeWindow( theWindow, LoWord(windowSize), HiWord(windowSize), true);
			SWChangeWorldRect( gSpriteWorldP, &theWindow->portRect, false);
			SWDrawTilesInBackground( gSpriteWorldP );
			SWUpdateScrollingSpriteWorld( gSpriteWorldP , false );
			
				// Reset data used by SmoothScrollingWorldMoveProc
			worldRect = gSpriteWorldP->windRect;
			OffsetRect(&worldRect, -worldRect.left, -worldRect.top);
			CenterRect( &gScreenMidRect, &worldRect );
			break;
		case inContent:
			break;
	}
}


///--------------------------------------------------------------------------------------
//  ShutDown (clean up and dispose of the SpriteWorld)
///--------------------------------------------------------------------------------------

void	ShutDown( void )
{
	SWDisposeSpriteWorld(&gSpriteWorldP);
	SWExitSpriteWorld();
	
	FlushEvents(everyEvent, 0);
	ShowCursor();
}


///--------------------------------------------------------------------------------------
//  TileChangeProc
///--------------------------------------------------------------------------------------

SW_FUNC void TileChangeProc(
	SpriteWorldPtr spriteWorldP)
{
	short			curImage;
	static short	wallDelay = 0, diamondDelay = 0;
	static short	oldTicks = 0;
	short			ticksPassed, ticks;
	
		// Initialize oldTicks the first time this function is called
	if (oldTicks == 0)
		oldTicks = TickCount();
	
	ticks = TickCount();
	ticksPassed = ticks - oldTicks;		// Number of ticks passed since last call
	oldTicks = ticks;
	
		// kWallTile
	wallDelay += ticksPassed;
	if (wallDelay >= kWallFrameRate)
	{
		curImage = spriteWorldP->curTileImage[kWallTile];
		if (curImage < kLastWallTile)
			curImage++;
		else
			curImage = kWallTile;
		
		SWChangeTileImage(spriteWorldP, kWallTile, curImage);
		wallDelay = 0;
	}
	
	
		// kDiamondTile
	diamondDelay += ticksPassed;
	if (diamondDelay >= kDiamondFrameRate)
	{
		curImage = spriteWorldP->curTileImage[kDiamondTile];
		if (curImage < kLastDiamondTile)
			curImage++;
		else
			curImage = kDiamondTile;
		
		SWChangeTileImage(spriteWorldP, kDiamondTile, curImage);
		diamondDelay = 0;
	}
}


///--------------------------------------------------------------------------------------
//  KeySpriteMoveProc
///--------------------------------------------------------------------------------------

SW_FUNC void KeySpriteMoveProc(SpritePtr srcSpriteP)
{
	short	row, col;
	short	rowDelta, colDelta;
	short	tile;
	
	
	row = srcSpriteP->destFrameRect.top / kTileHeight;
	col = srcSpriteP->destFrameRect.left / kTileWidth;

	if (row * kTileHeight == srcSpriteP->destFrameRect.top &&
		col * kTileWidth == srcSpriteP->destFrameRect.left)
	{
		tile = gTileMap[row][col];
		
			// Leave black trail behind sprite
		if (tile == kGrassTile || tile == kDiamondTile)
			SWDrawTile(gSpriteWorldP, 0, row, col, kBlackTile);
		else if (tile == kTunnelTile2 || tile == kTunnelTile5 || tile == kWireTile1)
			SWDrawTile(gSpriteWorldP, 0, row, col, tile+1);
			
		if (tile == kDiamondTile)
		{
			gNumDiamonds++;			// Increase number of diamonds sprite has collected
			UpdateDiamondMeter();	// Update meter
		}
		

		rowDelta = 0;
		colDelta = 0;
		
		if (gKeys.left)
			colDelta = -1;
		else if (gKeys.right)
			colDelta = 1;
		else if (gKeys.up)
			rowDelta = -1;
		else if (gKeys.down)
			rowDelta = 1;
		
				
		
		tile = gTileMap[row + rowDelta][col + colDelta];
		
		if (tile != kWallTile && tile != kTunnelTile1 && tile != kTunnelTile4)
		{
			srcSpriteP->vertMoveDelta = rowDelta * kSpriteMoveDelta;
			srcSpriteP->horizMoveDelta = colDelta * kSpriteMoveDelta;
		}
		else
		{
			srcSpriteP->vertMoveDelta = 0;
			srcSpriteP->horizMoveDelta = 0;
		}
	}
	
	SWOffsetSprite(srcSpriteP, srcSpriteP->horizMoveDelta, srcSpriteP->vertMoveDelta);
}


///--------------------------------------------------------------------------------------
//  MouseSpriteMoveProc
///--------------------------------------------------------------------------------------

SW_FUNC void MouseSpriteMoveProc(SpritePtr srcSpriteP)
{
	Point				mousePoint;
	
		// Get mouse coordinates local to the window
	SetPort(gWindowP);
	GetMouse(&mousePoint);
	
		// Subtract any offset from worldRect to window if a worldRect is used
	mousePoint.h -= gSpriteWorldP->windRect.left;
	mousePoint.v -= gSpriteWorldP->windRect.top;
	
	SWMoveSprite(srcSpriteP, 
		mousePoint.h - 1 + gSpriteWorldP->visScrollRect.left, 
		mousePoint.v - 1 + gSpriteWorldP->visScrollRect.top);
}


///--------------------------------------------------------------------------------------
//  DiamondMeterSpriteMoveProc - not installed as a MoveProc, but called directly
///--------------------------------------------------------------------------------------

SW_FUNC void DiamondMeterSpriteMoveProc(SpritePtr srcSpriteP)
{
		// Move sprite to top-left corner of current visScrollRect location
	SWMoveSprite(srcSpriteP, 
		gSpriteWorldP->visScrollRect.left + 10, 
		gSpriteWorldP->visScrollRect.top + 10);
}


///--------------------------------------------------------------------------------------
//  UpdateDiamondMeter - change the meter Sprite's image and mask
///--------------------------------------------------------------------------------------

SW_FUNC void UpdateDiamondMeter( void )
{
	double	percent;
	Rect	meterRect;
	
	percent = (double)gNumDiamonds / gNumDiamondsInMap;
	
	SetRect(&meterRect, 17, 2, (108 * percent) + 17, 14);
	
	
		// Set port to our sprite's framePort GWorld
	SetGWorld(gDiamondMeterSpriteP->curFrameP->framePort, nil);
	
	ForeColor(magentaColor);
	PaintRect(&meterRect);
	
	
		// Set port to our sprite's pixel mask GWorld
	SetGWorld(gDiamondMeterSpriteP->curFrameP->maskPort, nil);
	
		// Mask image is inverted when in 8-bit or less
	if (gSpriteWorldP->pixelDepth <= 8)
		ForeColor(whiteColor);
	else
		ForeColor(blackColor);
	
	PaintRect(&meterRect);
	
	
		// Set sprite to be redrawn, since we've changed its image
	gDiamondMeterSpriteP->needsToBeDrawn = true;
}


///--------------------------------------------------------------------------------------
//  SmoothScrollingWorldMoveProc - our scrolling WorldMoveProc
///--------------------------------------------------------------------------------------

SW_FUNC void SmoothScrollingWorldMoveProc(
	SpriteWorldPtr spriteWorldP,
	SpritePtr followSpriteP)
{	
	short	screenMidRectTop, screenMidRectLeft;
	
	screenMidRectTop = gScreenMidRect.top + spriteWorldP->visScrollRect.top;
	screenMidRectLeft = gScreenMidRect.left + spriteWorldP->visScrollRect.left;
	
	
	spriteWorldP->horizScrollDelta = (kSpriteMoveDelta * 
		(followSpriteP->destFrameRect.left - screenMidRectLeft) ) / kSpriteMoveDistance;
	
	spriteWorldP->vertScrollDelta = (kSpriteMoveDelta * 
		(followSpriteP->destFrameRect.top - screenMidRectTop) ) / kSpriteMoveDistance;
	
	if (kInterlacedMode)
		spriteWorldP->vertScrollDelta = spriteWorldP->vertScrollDelta>>1<<1;
}

